#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <axis/client/Call.hpp>

AXIS_CPP_NAMESPACE_USE

extern "C" {

#include <axis/GDefine.h>

#include <axis/AxisUserAPI.h>

#include <axis/SoapEnvVersions.h>

#include <axis/TypeMapping.h>

#include <axis/WSDDDefines.h>

#include <axis/client/Call.h>

AXISC_STORAGE_CLASS_INFO AXISCHANDLE axiscCreateCall() {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscCreateCall", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = new Call();
		{
		#ifdef ENABLE_AXISTRACE
			AXISCHANDLE traceRet = ((AXISCHANDLE)c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscCreateCall", NULL, 0,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (AXISCHANDLE)c;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscDestroyCall(AXISCHANDLE call) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscDestroyCall", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	delete c;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscDestroyCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscSetSOAPVersionCall(AXISCHANDLE call, AXISC_SOAP_VERSION version) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetSOAPVersionCall", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_DATA, sizeof(AXISC_SOAP_VERSION), ((void*)&version));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	c->setSOAPVersion((SOAP_VERSION)version);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetSOAPVersionCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO int axiscSetTransportPropertyCall(AXISCHANDLE call, AXISC_TRANSPORT_INFORMATION_TYPE type, 
	const char * value) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetTransportPropertyCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_DATA, sizeof(AXISC_TRANSPORT_INFORMATION_TYPE), ((void*)&type),
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->setTransportProperty((AXIS_TRANSPORT_INFORMATION_TYPE)type, value));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetTransportPropertyCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->setTransportProperty((AXIS_TRANSPORT_INFORMATION_TYPE)type, value);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO const char * axiscGetTransportPropertyCall(AXISCHANDLE call, const char *key, 
	AxiscBool response) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetTransportPropertyCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_STRING, 0, ((void*)&key),
					TRACETYPE_DATA, sizeof(AxiscBool), ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (c->getTransportProperty(key, (bool)(response !=0 )));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetTransportPropertyCall", NULL, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getTransportProperty(key, (bool)(response !=0 ));
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSetHandlerPropertyCall(AXISCHANDLE call, AxiscChar * name, 
	void * value, int len) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetHandlerPropertyCall", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&name),
					TRACETYPE_POINTER, 1, ((void*)&value),
					TRACETYPE_INT, 0, ((void*)&len));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->setHandlerProperty(name,value,len));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetHandlerPropertyCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->setHandlerProperty(name,value,len);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSetProtocolCall(AXISCHANDLE call, AXISC_PROTOCOL_TYPE protocol) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetProtocolCall", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_DATA, sizeof(AXISC_PROTOCOL_TYPE), ((void*)&protocol));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->setProtocol((AXIS_PROTOCOL_TYPE)protocol));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetProtocolCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->setProtocol((AXIS_PROTOCOL_TYPE)protocol);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO AXISC_PROTOCOL_TYPE axiscGetProtocol(AXISCHANDLE call) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetProtocol", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			AXISC_PROTOCOL_TYPE traceRet = ((AXISC_PROTOCOL_TYPE)(c->getProtocol()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetProtocol", NULL, 0,
					TRACETYPE_DATA, sizeof(AXISC_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (AXISC_PROTOCOL_TYPE)(c->getProtocol());
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscUnInitializeCall(AXISCHANDLE call) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscUnInitializeCall", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->unInitialize());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscUnInitializeCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->unInitialize();
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscInitializeCall(AXISCHANDLE call, AXISC_PROVIDERTYPE nStyle) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscInitializeCall", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_DATA, sizeof(AXISC_PROVIDERTYPE), ((void*)&nStyle));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->initialize((PROVIDERTYPE)nStyle));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscInitializeCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->initialize((PROVIDERTYPE)nStyle);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscInvokeCall(AXISCHANDLE call) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscInvokeCall", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->invoke());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscInvokeCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->invoke();
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscAddCmplxParameterCall(AXISCHANDLE call, void * pObject, 
	void * pSZFunct, void * pDelFunct, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddCmplxParameterCall", NULL, 6,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, 1, ((void*)&pObject),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	c->addCmplxParameter(pObject,pSZFunct,pDelFunct,pName,pNamespace);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddCmplxParameterCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscAddCmplxArrayParameterCall(AXISCHANDLE call, Axisc_Array * pArray, 
	void * pSZFunct, void * pDelFunct, void * pSizeFunct, 
	const AxiscChar * pName, const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddCmplxArrayParameterCall", NULL, 7,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	c->addCmplxArrayParameter((Axis_Array*)pArray,pSZFunct,pDelFunct,pSizeFunct,pName,pNamespace);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddCmplxArrayParameterCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscAddBasicArrayParameterCall(AXISCHANDLE call, Axisc_Array * pArray, 
	AXISC_XSDTYPE nType, const AxiscChar * pName) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddBasicArrayParameterCall", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&pArray),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&nType),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	c->addBasicArrayParameter((Axis_Array*)pArray, (XSDTYPE)nType, pName);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddBasicArrayParameterCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscAddParameterCall(AXISCHANDLE call, void * pValue, 
	const char * pchName, AXISC_XSDTYPE nType) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddParameterCall", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_STRING, 0, ((void*)&pchName),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&nType));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	c->addParameter(pValue,pchName,(XSDTYPE)nType);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddParameterCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscSetOperationCall(AXISCHANDLE call, const char * pchOperation, 
	const char * pchNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetOperationCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_STRING, 0, ((void*)&pchOperation),
					TRACETYPE_STRING, 0, ((void*)&pchNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	c->setOperation(pchOperation,pchNamespace);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetOperationCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO int axiscSetEndpointURICall(AXISCHANDLE call, const char * pchEndpointURI) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetEndpointURICall", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_STRING, 0, ((void*)&pchEndpointURI));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->setEndpointURI(pchEndpointURI));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetEndpointURICall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->setEndpointURI(pchEndpointURI);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO AXISCHANDLE axiscCreateHeaderBlockCall(AXISCHANDLE call, AxiscChar * pachLocalName, 
	AxiscChar * pachUri, AxiscChar * pachPrefix) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscCreateHeaderBlockCall", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pachLocalName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pachUri),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pachPrefix));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			AXISCHANDLE traceRet = ((AXISCHANDLE)(c->createHeaderBlock(pachLocalName, pachUri, pachPrefix)));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscCreateHeaderBlockCall", NULL, 0,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (AXISCHANDLE)(c->createHeaderBlock(pachLocalName, pachUri, pachPrefix));
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__int * axiscGetElementAsIntCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsIntCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__int* traceRet = (c->getElementAsInt(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsIntCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__int), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsInt(pName,pNamespace);
		#endif
	}

}

/*AXISC_STORAGE_CLASS_INFO int axiscGetFaultDetailCall(AXISCHANDLE call, char * * ppcDetail) {
	Call *c = (Call*)call;
	return c->getFaultDetail(ppcDetail);
}*/

AXISC_STORAGE_CLASS_INFO xsdc__boolean * axiscGetElementAsBooleanCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsBooleanCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__boolean* traceRet = ((xsdc__boolean* )(c->getElementAsBoolean(pName,pNamespace)));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsBooleanCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__boolean), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (xsdc__boolean* )(c->getElementAsBoolean(pName,pNamespace));
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedInt * axiscGetElementAsUnsignedIntCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsUnsignedIntCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedInt* traceRet = (c->getElementAsUnsignedInt(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsUnsignedIntCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedInt), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsUnsignedInt(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__short * axiscGetElementAsShortCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsShortCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__short* traceRet = (c->getElementAsShort(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsShortCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__short), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsShort(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedShort * axiscGetElementAsUnsignedShortCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsUnsignedShortCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedShort* traceRet = (c->getElementAsUnsignedShort(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsUnsignedShortCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedShort), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsUnsignedShort(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__byte * axiscGetElementAsByteCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsByteCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__byte* traceRet = (c->getElementAsByte(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsByteCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__byte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsByte(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedByte * axiscGetElementAsUnsignedByteCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsUnsignedByteCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedByte* traceRet = (c->getElementAsUnsignedByte(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsUnsignedByteCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedByte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsUnsignedByte(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__long * axiscGetElementAsLongCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsLongCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__long* traceRet = ((xsd__long*)(c->getElementAsLong(pName,pNamespace)));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsLongCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__long), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (xsd__long*)(c->getElementAsLong(pName,pNamespace));
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__integer * axiscGetElementAsIntegerCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsIntegerCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__integer* traceRet = (c->getElementAsInteger(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsIntegerCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__integer), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsInteger(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedLong * axiscGetElementAsUnsignedLongCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsUnsignedLongCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedLong* traceRet = (c->getElementAsUnsignedLong(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsUnsignedLongCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedLong), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsUnsignedLong(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__float * axiscGetElementAsFloatCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsFloatCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__float* traceRet = (c->getElementAsFloat(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsFloatCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__float), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsFloat(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__double * axiscGetElementAsDoubleCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsDoubleCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__double* traceRet = (c->getElementAsDouble(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsDoubleCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__double), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsDouble(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__decimal * axiscGetElementAsDecimalCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsDecimalCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__decimal* traceRet = (c->getElementAsDecimal(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsDecimalCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__decimal), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsDecimal(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__string axiscGetElementAsStringCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsStringCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__string traceRet = (c->getElementAsString(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsStringCall", NULL, 0,
					TRACETYPE_DATA, sizeof(xsdc__string), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsString(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__anyURI axiscGetElementAsAnyURICall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsAnyURICall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__anyURI traceRet = (c->getElementAsAnyURI(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsAnyURICall", NULL, 0,
					TRACETYPE_DATA, sizeof(xsdc__anyURI), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsAnyURI(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__QName axiscGetElementAsQNameCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsQNameCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__QName traceRet = (c->getElementAsQName(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsQNameCall", NULL, 0,
					TRACETYPE_DATA, sizeof(xsdc__QName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsQName(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__hexBinary * axiscGetElementAsHexBinaryCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsHexBinaryCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	//TODO: Unimplemented
	xsdc__hexBinary * hb = new xsdc__hexBinary();
	memset(hb,0,sizeof(hb));
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__hexBinary* traceRet = (hb);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsHexBinaryCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return hb;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__base64Binary * axiscGetElementAsBase64BinaryCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsBase64BinaryCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	//TODO: Unimplemented
	xsdc__base64Binary * bb = new xsdc__base64Binary();
	memset(bb,0,sizeof(bb));
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__base64Binary* traceRet = (bb);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsBase64BinaryCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return bb;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__dateTime * axiscGetElementAsDateTimeCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsDateTimeCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__dateTime* traceRet = (c->getElementAsDateTime(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsDateTimeCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__dateTime), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsDateTime(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__date * axiscGetElementAsDateCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsDateCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__date* traceRet = (c->getElementAsDate(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsDateCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__date), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsDate(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__time * axiscGetElementAsTimeCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsTimeCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__time* traceRet = (c->getElementAsTime(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsTimeCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__time), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsTime(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__duration * axiscGetElementAsDurationCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetElementAsDurationCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__duration* traceRet = (c->getElementAsDuration(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetElementAsDurationCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__duration), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getElementAsDuration(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__int * axiscGetAttributeAsIntCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsIntCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__int* traceRet = (c->getAttributeAsInt(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsIntCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__int), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsInt(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__boolean * axiscGetAttributeAsBooleanCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsBooleanCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__boolean* traceRet = ((xsdc__boolean* )(c->getAttributeAsBoolean(pName,pNamespace)));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsBooleanCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__boolean), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (xsdc__boolean* )(c->getAttributeAsBoolean(pName,pNamespace));
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedInt * axiscGetAttributeAsUnsignedIntCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsUnsignedIntCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedInt* traceRet = (c->getAttributeAsUnsignedInt(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsUnsignedIntCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedInt), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsUnsignedInt(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__short * axiscGetAttributeAsShortCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsShortCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__short* traceRet = (c->getAttributeAsShort(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsShortCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__short), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsShort(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedShort * axiscGetAttributeAsUnsignedShortCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsUnsignedShortCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedShort* traceRet = (c->getAttributeAsUnsignedShort(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsUnsignedShortCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedShort), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsUnsignedShort(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__byte * axiscGetAttributeAsByteCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsByteCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__byte* traceRet = (c->getAttributeAsByte(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsByteCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__byte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsByte(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedByte * axiscGetAttributeAsUnsignedByteCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsUnsignedByteCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedByte* traceRet = (c->getAttributeAsUnsignedByte(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsUnsignedByteCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedByte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsUnsignedByte(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__long * axiscGetAttributeAsLongCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsLongCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__long* traceRet = (c->getAttributeAsLong(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsLongCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__long), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsLong(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__integer * axiscGetAttributeAsIntegerCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsIntegerCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__integer* traceRet = (c->getAttributeAsInteger(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsIntegerCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__integer), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsInteger(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__unsignedLong * axiscGetAttributeAsUnsignedLongCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsUnsignedLongCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__unsignedLong* traceRet = (c->getAttributeAsUnsignedLong(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsUnsignedLongCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__unsignedLong), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsUnsignedLong(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__float * axiscGetAttributeAsFloatCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsFloatCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__float* traceRet = (c->getAttributeAsFloat(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsFloatCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__float), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsFloat(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__double * axiscGetAttributeAsDoubleCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsDoubleCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__double* traceRet = (c->getAttributeAsDouble(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsDoubleCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__double), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsDouble(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__decimal * axiscGetAttributeAsDecimalCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsDecimalCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__decimal* traceRet = (c->getAttributeAsDecimal(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsDecimalCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__decimal), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsDecimal(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__string axiscGetAttributeAsStringCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsStringCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__string traceRet = (c->getAttributeAsString(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsStringCall", NULL, 0,
					TRACETYPE_DATA, sizeof(xsdc__string), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsString(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__anyURI axiscGetAttributeAsAnyURICall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsAnyURICall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__anyURI traceRet = (c->getAttributeAsAnyURI(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsAnyURICall", NULL, 0,
					TRACETYPE_DATA, sizeof(xsdc__anyURI), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsAnyURI(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__QName axiscGetAttributeAsQNameCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsQNameCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__QName traceRet = (c->getAttributeAsQName(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsQNameCall", NULL, 0,
					TRACETYPE_DATA, sizeof(xsdc__QName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsQName(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__hexBinary * axiscGetAttributeAsHexBinaryCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsHexBinaryCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	//TODO: Unimplemented
	xsdc__hexBinary * hb = new xsdc__hexBinary();
	memset(hb,0,sizeof(hb));
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__hexBinary* traceRet = (hb);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsHexBinaryCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return hb;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__base64Binary * axiscGetAttributeAsBase64BinaryCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsBase64BinaryCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	//TODO: Unimplemented
	xsdc__base64Binary *bb = new xsdc__base64Binary();
	memset(bb,0,sizeof(bb));
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__base64Binary* traceRet = (bb);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsBase64BinaryCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return bb;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__dateTime * axiscGetAttributeAsDateTimeCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsDateTimeCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__dateTime* traceRet = (c->getAttributeAsDateTime(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsDateTimeCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__dateTime), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsDateTime(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__date * axiscGetAttributeAsDateCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsDateCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__date* traceRet = (c->getAttributeAsDate(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsDateCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__date), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsDate(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__time * axiscGetAttributeAsTimeCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsTimeCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__time* traceRet = (c->getAttributeAsTime(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsTimeCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__time), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsTime(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO xsdc__duration * axiscGetAttributeAsDurationCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAttributeAsDurationCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			xsdc__duration* traceRet = (c->getAttributeAsDuration(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAttributeAsDurationCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(xsdc__duration), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getAttributeAsDuration(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void * axiscGetCmplxObjectCall(AXISCHANDLE call, void * pDZFunct, 
	void * pCreFunct, void * pDelFunct, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetCmplxObjectCall", NULL, 6,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (c->getCmplxObject(pDZFunct,pCreFunct,pDelFunct,pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetCmplxObjectCall", NULL, 0,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getCmplxObject(pDZFunct,pCreFunct,pDelFunct,pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO Axisc_Array* axiscGetCmplxArrayCall(AXISCHANDLE call, Axisc_Array* pArray, void * pDZFunct, 
	void * pCreFunct, void * pDelFunct, void * pSizeFunct, 
	const AxiscChar * pName, const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetCmplxArrayCall", NULL, 8,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	//TODO: Unimplemented
	Axisc_Array* aa = new Axisc_Array();
	memset(aa,0,sizeof(aa));
		{
		#ifdef ENABLE_AXISTRACE
			Axisc_Array* traceRet = (aa);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetCmplxArrayCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return aa;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO Axisc_Array* axiscGetBasicArrayCall(AXISCHANDLE call, AXISC_XSDTYPE nType, 
	const AxiscChar * pName, const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetBasicArrayCall", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&nType),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	//TODO: Unimplemented
	Axisc_Array* aa = new Axisc_Array();
	memset(aa,0,sizeof(aa));
		{
		#ifdef ENABLE_AXISTRACE
			Axisc_Array* traceRet = (aa);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetBasicArrayCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return aa;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscCheckMessageCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscCheckMessageCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->checkMessage(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscCheckMessageCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->checkMessage(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void * axiscCheckFaultCall(AXISCHANDLE call, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscCheckFaultCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (c->checkFault(pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscCheckFaultCall", NULL, 0,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->checkFault(pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscGetStatusCall(AXISCHANDLE call) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetStatusCall", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->getStatus());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetStatusCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getStatus();
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscSetProxyCall(AXISCHANDLE call, const char * pcProxyHost, 
	unsigned int uiProxyPort) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetProxyCall", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_STRING, 0, ((void*)&pcProxyHost),
					TRACETYPE_UINT, 0, ((void*)&uiProxyPort));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
	c->setProxy(pcProxyHost,uiProxyPort);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetProxyCall", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO AxiscAnyType * axiscGetAnyObjectCall(AXISCHANDLE call) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetAnyObjectCall", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			AxiscAnyType* traceRet = ((AxiscAnyType*)(c->getAnyObject()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetAnyObjectCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(AxiscAnyType), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (AxiscAnyType*)(c->getAnyObject());
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscAddAnyObjectCall(AXISCHANDLE call, AxiscAnyType * pAnyObject) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddAnyObjectCall", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscAnyType), ((void*)&pAnyObject));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c->addAnyObject((AnyType*)pAnyObject));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddAnyObjectCall", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->addAnyObject((AnyType*)pAnyObject);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO const AxiscChar * axiscGetNamespacePrefixCall(AXISCHANDLE call, const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetNamespacePrefixCall", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	Call *c = (Call*)call;
		{
		#ifdef ENABLE_AXISTRACE
			const AxiscChar* traceRet = (c->getNamespacePrefix(pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetNamespacePrefixCall", NULL, 0,
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c->getNamespacePrefix(pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscSetSOAPMethodAttribute(AXISCHANDLE call, const AxiscChar * pLocalname, 
    const AxiscChar * pPrefix, const AxiscChar * pUri, 
   const AxiscChar * pValue) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetSOAPMethodAttribute", NULL, 5,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&call),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pLocalname),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pPrefix),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pUri),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pValue));	  /* AUTOINSERTED TRACE */
	#endif

   Call *c = (Call*)call;
    c->setSOAPMethodAttribute(pLocalname,pPrefix,pUri,pValue);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetSOAPMethodAttribute", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

}

