#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2005 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 *   @author John Kelly (john_kelly@uk.ibm.com)
 *
 */

#include "../platforms/PlatformAutoSense.hpp"

#include "AxisCPPConfigDefaultsImpl.hpp"

#include "AxisUtils.h"

#include "AxisConfig.h"

#include "AxisConfigException.h"

#include "../common/AxisTrace.h"

#include <stdlib.h>

#include <iostream>

#include <string.h>

extern AXIS_CPP_NAMESPACE_PREFIX AxisConfig* g_pConfig;

AXIS_CPP_NAMESPACE_START 
AxisCPPConfigDefaultsImpl::AxisCPPConfigDefaultsImpl ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "AxisCPPConfigDefaultsImpl", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  m_pcClientLog = "";
  m_pcClientWSDDFile = "";
  m_pcXMLParserLibrary = "";
  m_pcHTTPTransportLibrary = "";
  m_pcHTTPChannelLibrary = "";
  m_pcHTTPSSLChannelLibrary = "";
  m_pcNodeName = "";
  m_pcListenPort = "";
  m_pcSecureInfo = "";
  m_pcAxisHome = "";
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "AxisCPPConfigDefaultsImpl", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AxisCPPConfigDefaultsImpl::~AxisCPPConfigDefaultsImpl ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "~AxisCPPConfigDefaultsImpl", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "~AxisCPPConfigDefaultsImpl", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::apply ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "apply", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  /* only allow user to apply once befor initialization and NOT AT ALL after */
  if(!g_pConfig)
  {
      char *pszValue=0;
      g_pConfig = new AxisConfig ();
      //todo set each value, if set
      pszValue = getClientLog();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_CLIENTLOGPATH, pszValue);
      pszValue = getClientWSDDFile();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_WSDDFILEPATH, pszValue);
      pszValue = getXMLParserLibrary();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_XMLPARSER, pszValue);
      pszValue = getHTTPTransportLibrary();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_TRANSPORTHTTP, pszValue);
      pszValue = getHTTPChannelLibrary();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_CHANNEL_HTTP, pszValue);
      pszValue = getHTTPSSLChannelLibrary();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_SSLCHANNEL_HTTP, pszValue);
      pszValue = getNodeName();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_NODENAME, pszValue);
      pszValue = getListenPort();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_LISTENPORT, pszValue);
      pszValue = getSecureInfo();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_SECUREINFO, pszValue);
      pszValue = getAxisHome();
      if(pszValue)
          g_pConfig->setValue(0,AXCONF_AXISHOME, pszValue);
  }
  else
  {
      AXISTRACE1 ("configuration defaults aleady set", WARN);
      throw AxisConfigException(CONFIG_DEFAULTS_ALREADY_SET);
  }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "apply", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setClientLog (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setClientLog", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcClientLog = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setClientLog", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setClientWSDDFile (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setClientWSDDFile", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcClientWSDDFile = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setClientWSDDFile", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setXMLParserLibrary (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setXMLParserLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcXMLParserLibrary = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setXMLParserLibrary", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setHTTPTransportLibrary (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setHTTPTransportLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcHTTPTransportLibrary = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setHTTPTransportLibrary", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setHTTPChannelLibrary (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setHTTPChannelLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcHTTPChannelLibrary = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setHTTPChannelLibrary", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setHTTPSSLChannelLibrary (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setHTTPSSLChannelLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcHTTPSSLChannelLibrary = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setHTTPSSLChannelLibrary", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setNodeName (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setNodeName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcNodeName = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setNodeName", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setListenPort (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setListenPort", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcListenPort = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setListenPort", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setSecureInfo (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setSecureInfo", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcSecureInfo = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setSecureInfo", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void AxisCPPConfigDefaultsImpl::setAxisHome (char *value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "setAxisHome", this, 1,
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    m_pcAxisHome = value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "setAxisHome", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

char* AxisCPPConfigDefaultsImpl::getClientLog ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getClientLog", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcClientLog.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getClientLog", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcClientLog.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getClientLog", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcClientLog.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getClientWSDDFile ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getClientWSDDFile", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcClientWSDDFile.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getClientWSDDFile", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcClientWSDDFile.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getClientWSDDFile", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcClientWSDDFile.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getXMLParserLibrary ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getXMLParserLibrary", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcXMLParserLibrary.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getXMLParserLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcXMLParserLibrary.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getXMLParserLibrary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcXMLParserLibrary.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getHTTPTransportLibrary ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getHTTPTransportLibrary", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcHTTPTransportLibrary.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getHTTPTransportLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcHTTPTransportLibrary.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getHTTPTransportLibrary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcHTTPTransportLibrary.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getHTTPChannelLibrary ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getHTTPChannelLibrary", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcHTTPChannelLibrary.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getHTTPChannelLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcHTTPChannelLibrary.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getHTTPChannelLibrary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcHTTPChannelLibrary.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getHTTPSSLChannelLibrary ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getHTTPSSLChannelLibrary", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcHTTPSSLChannelLibrary.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getHTTPSSLChannelLibrary", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcHTTPSSLChannelLibrary.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getHTTPSSLChannelLibrary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcHTTPSSLChannelLibrary.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getNodeName ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getNodeName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcNodeName.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getNodeName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcNodeName.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getNodeName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcNodeName.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getListenPort ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getListenPort", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcListenPort.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getListenPort", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcListenPort.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getListenPort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcListenPort.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getSecureInfo ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getSecureInfo", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcSecureInfo.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getSecureInfo", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcSecureInfo.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getSecureInfo", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcSecureInfo.c_str());
		#endif
	}

}

char* AxisCPPConfigDefaultsImpl::getAxisHome ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisCPPConfigDefaultsImpl", "getAxisHome", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == m_pcAxisHome.length()) 	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getAxisHome", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((char*)(m_pcAxisHome.c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisCPPConfigDefaultsImpl", "getAxisHome", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (char*)(m_pcAxisHome.c_str());
		#endif
	}

}

/** /todo place some sample code here
*/

AXIS_CPP_NAMESPACE_END

