#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

#include "DeserializerPool.h"

#include "../common/AxisTrace.h"

AXIS_CPP_NAMESPACE_START

using namespace std;

DeserializerPool::DeserializerPool()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("DeserializerPool", "DeserializerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	m_DZList.clear();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("DeserializerPool", "DeserializerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 * ~DeserializerPool sometimes fails on AIX, so this method has been
 * recoded to make it more defensive and added extra trace. It fails
 * intermittently and the last line in the trace shows the exit of
 * the last ~SoapSerializer().
 * 
 * 
 */

DeserializerPool::~DeserializerPool ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("DeserializerPool", "~DeserializerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    list<IWrapperSoapDeSerializer*>::iterator it = m_DZList.begin();
	while (it != m_DZList.end())
    {
		IWrapperSoapDeSerializer *dz = *it;
#ifdef ENABLE_AXISTRACE
		if (AxisTrace::isTraceOn()) 
		{
			char text[256];
			sprintf(text, "~DeserializerPool<%p> dz=%p", this, dz);
			AxisTrace::traceLine(text);
		}
#endif
		if (NULL != dz)
	        delete dz;
		it++;
    }
	m_DZList.clear();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("DeserializerPool", "~DeserializerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int DeserializerPool::getInstance (IWrapperSoapDeSerializer** ppDZ)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("DeserializerPool", "getInstance", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapDeSerializer*), ((void*)&ppDZ));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    if (!m_DZList.empty ())
    {
        *ppDZ = m_DZList.front ();
        m_DZList.pop_front ();
    }
    else
    {
        *ppDZ = new SoapDeSerializer ();
        if (AXIS_SUCCESS != ((SoapDeSerializer*)(*ppDZ))->init ())
        {
            delete* ppDZ;
            *ppDZ = NULL;
            //unlock ();
            AXISTRACE1 ("Deserializer could not be initialized", CRITICAL);
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("DeserializerPool", "getInstance", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

        }
    }
    //unlock ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("DeserializerPool", "getInstance", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int DeserializerPool::putInstance (IWrapperSoapDeSerializer* pDZ)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("DeserializerPool", "putInstance", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapDeSerializer), ((void*)&pDZ));	  /* AUTOINSERTED TRACE */
	#endif

    if (AXIS_SUCCESS != ((SoapDeSerializer*)pDZ)->init ())
    {
        delete pDZ;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("DeserializerPool", "putInstance", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    }
    //lock ();
	Lock l(this);
    m_DZList.push_back (pDZ);
    //unlock ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("DeserializerPool", "putInstance", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

