#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

/*
 * Revision 1.1  2004/08/26 roshan
 * Added code to the putInstance method, so that it calls the "fini" method of
 *  a Handler when the handler is released
 */

#include "RequestScopeHandlerPool.h"

#include "HandlerLoader.h"

extern AXIS_CPP_NAMESPACE_PREFIX HandlerLoader* g_pHandlerLoader;

AXIS_CPP_NAMESPACE_START

RequestScopeHandlerPool::RequestScopeHandlerPool ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("RequestScopeHandlerPool", "RequestScopeHandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("RequestScopeHandlerPool", "RequestScopeHandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

RequestScopeHandlerPool::~RequestScopeHandlerPool ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("RequestScopeHandlerPool", "~RequestScopeHandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    for (map < int, list <BasicHandler*> >::iterator it =
        m_Handlers.begin (); it != m_Handlers.end (); it++)
    {
        for (list <BasicHandler*>::iterator itr = (*it).second.begin ();
            itr != (*it).second.end (); itr++)
        {
            g_pHandlerLoader->deleteHandler (*itr, (*it).first);
        }
        (*it).second.clear ();
    }
    m_Handlers.clear ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("RequestScopeHandlerPool", "~RequestScopeHandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int RequestScopeHandlerPool::getInstance (BasicHandler** pHandler, int nLibId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("RequestScopeHandlerPool", "getInstance", this, 2,
					TRACETYPE_POINTER, sizeof(BasicHandler*), ((void*)&pHandler),
					TRACETYPE_INT, 0, ((void*)&nLibId));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    int Status;
    if (m_Handlers.find (nLibId) != m_Handlers.end ())
    {
        if (m_Handlers[nLibId].empty ())
        {
            Status = g_pHandlerLoader->createHandler (pHandler, nLibId);
            if (AXIS_SUCCESS == Status)
            {
                /* This just creates the entry in m_Handlers so that next 
	         * time we know that the DLL is loaded
		 */ 
                m_Handlers[nLibId].clear ();
            }
            //unlock ();
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("RequestScopeHandlerPool", "getInstance", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

        }
        else
        {
            *pHandler = m_Handlers[nLibId].front ();
            m_Handlers[nLibId].pop_front ();
            //unlock ();
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("RequestScopeHandlerPool", "getInstance", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

        }
    }
    else // Not even the handler DLL loaded
    {
        Status = g_pHandlerLoader->createHandler (pHandler, nLibId);
        if (AXIS_SUCCESS == Status)
        {
            /* This just creates the entry in m_Handlers so that next time we 
	     * know that the DLL is loaded
	     */ 
            m_Handlers[nLibId].clear ();
        }
        //unlock ();
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("RequestScopeHandlerPool", "getInstance", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    }
}

int RequestScopeHandlerPool::putInstance (BasicHandler* pHandler, int nLibId, bool bWebService)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("RequestScopeHandlerPool", "putInstance", this, 3,
					TRACETYPE_POINTER, sizeof(BasicHandler), ((void*)&pHandler),
					TRACETYPE_INT, 0, ((void*)&nLibId),
					TRACETYPE_BOOL, 0, ((void*)&bWebService));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    if (0 != pHandler->_functions)
    {
        /* C Handler */
    }
    else if (0 != pHandler->_object)
    {
         if (!bWebService)
             ((HandlerBase *)(pHandler->_object))->fini();
    }
    m_Handlers[nLibId].push_back (pHandler);
    //unlock ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("RequestScopeHandlerPool", "putInstance", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

