#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

#include "SerializerPool.h"

#include "../common/AxisTrace.h"

AXIS_CPP_NAMESPACE_START

SerializerPool::SerializerPool ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SerializerPool", "SerializerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SerializerPool", "SerializerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SerializerPool::~SerializerPool ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SerializerPool", "~SerializerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    for (list <IWrapperSoapSerializer*>::iterator it = m_SZList.begin ();
         it != m_SZList.end (); it++)
    {
        delete (*it);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SerializerPool", "~SerializerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

// Pooling should be implemented

int SerializerPool::getInstance (IWrapperSoapSerializer** ppSZ)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SerializerPool", "getInstance", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapSerializer*), ((void*)&ppSZ));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    if (!m_SZList.empty ())
    {
        *ppSZ = m_SZList.front ();
        m_SZList.pop_front ();
    }
    else
    {
#ifdef USER_SERIALIZER
        *ppSZ = (IWrapperSoapSerializer*)GetUserSerializer();
#else
        *ppSZ = new SoapSerializer ();
#endif
        
    }
    if (AXIS_SUCCESS != ((SoapSerializer*)(*ppSZ))->init ())
    {
        delete *ppSZ;
        *ppSZ = NULL;
        //unlock ();
        AXISTRACE1 ("Serializer pool could not be initialized", CRITICAL);
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SerializerPool", "getInstance", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    }
    //unlock ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SerializerPool", "getInstance", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int SerializerPool::putInstance (IWrapperSoapSerializer* pSZ)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SerializerPool", "putInstance", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapSerializer), ((void*)&pSZ));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    m_SZList.push_back (pSZ);
    //unlock ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SerializerPool", "putInstance", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

