#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/**
 * This class 
 *
 * @author Susantha Kumara (skumara@virtusa.com, susantha@opensource.lk)
 *
 */

#include "../platforms/PlatformAutoSense.hpp"

#include "XMLParserFactory.h"

#include "../xml/XMLParser.h"

#include <stdio.h>

#include "../common/AxisConfig.h"

#include "AxisEngineException.h"

#include "../common/AxisTrace.h"
                                                                                                                             
extern AXIS_CPP_NAMESPACE_PREFIX AxisConfig* g_pConfig;

AXIS_CPP_NAMESPACE_START

const char* XMLParserFactory::m_pcLibraryPath = 0;

DLHandler XMLParserFactory::m_LibHandler = 0;

CREATE_OBJECT2 XMLParserFactory::m_Create = 0;

DELETE_OBJECT2 XMLParserFactory::m_Delete = 0;

XMLParserFactory::XMLParserFactory()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "XMLParserFactory", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	m_LibHandler = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "XMLParserFactory", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XMLParserFactory::~XMLParserFactory()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "~XMLParserFactory", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "~XMLParserFactory", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int XMLParserFactory::initialize()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "initialize", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pcLibraryPath = g_pConfig->getAxisConfProperty(AXCONF_XMLPARSER);
	if (!loadLib())
	{
        m_Create = (CREATE_OBJECT2) PLATFORM_GETPROCADDR(m_LibHandler, CREATE_FUNCTION2);
        m_Delete = (DELETE_OBJECT2) PLATFORM_GETPROCADDR(m_LibHandler, DELETE_FUNCTION2);
        if (!m_Create || !m_Delete)
        {
            unloadLib();
            AXISTRACE1("SERVER_ENGINE_LOADING_PARSER_FAILED" , CRITICAL);
            char *s = new char[strlen(m_pcLibraryPath)+1];
            strcpy(s,m_pcLibraryPath);
            throw AxisEngineException(SERVER_ENGINE_LOADING_PARSER_FAILED, s);
        } 
	  else
	  {
#ifdef ENABLE_AXISTRACE
            // Load function to do lib level inits
            INIT_OBJECT2 initializeLibrary;
            initializeLibrary = (INIT_OBJECT2) PLATFORM_GETPROCADDR(m_LibHandler, INIT_FUNCTION2);
            if (initializeLibrary)
                 (*initializeLibrary)(AxisTrace::getTraceEntrypoints());
#endif
	  }
	}
	else
	{
        AXISTRACE1("SERVER_ENGINE_LOADING_PARSER_FAILED" , CRITICAL);
        char *s = new char[strlen(m_pcLibraryPath)+1];
        strcpy(s,m_pcLibraryPath);
        throw AxisEngineException(SERVER_ENGINE_LOADING_PARSER_FAILED, s);
	}
   	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "initialize", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int XMLParserFactory::uninitialize()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "uninitialize", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

#ifdef ENABLE_AXISTRACE
      UNINIT_OBJECT2 uninitializeLibrary;
      uninitializeLibrary = (UNINIT_OBJECT2) PLATFORM_GETPROCADDR(m_LibHandler, UNINIT_FUNCTION2);
      if (uninitializeLibrary)
          (*uninitializeLibrary)();
#endif
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (unloadLib());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "uninitialize", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return unloadLib();
		#endif
	}

}

/**
 * Should create an instance of transport of type given by eProtocol
 */

XMLParser* XMLParserFactory::getParserObject()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "getParserObject", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

	XMLParser* pTpt = 0;
	if (m_Create) m_Create(&pTpt);
		{
		#ifdef ENABLE_AXISTRACE
			XMLParser* traceRet = (pTpt);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "getParserObject", NULL, 0,
					TRACETYPE_POINTER, sizeof(XMLParser), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pTpt;
		#endif
	}

}

void XMLParserFactory::destroyParserObject(XMLParser* pObject)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "destroyParserObject", NULL, 1,
					TRACETYPE_POINTER, sizeof(XMLParser), ((void*)&pObject));	  /* AUTOINSERTED TRACE */
	#endif

	m_Delete(pObject);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "destroyParserObject", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int XMLParserFactory::loadLib()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "loadLib", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_LibHandler = PLATFORM_LOADLIB(m_pcLibraryPath);
    if (!m_LibHandler)
    {
        AXISTRACE1("SERVER_ENGINE_LOADING_PARSER_FAILED" , CRITICAL);
        long dwError = GETLASTERROR
        string *    message = PLATFORM_GET_ERROR_MESSAGE( dwError);
        char        fullMessage[1024];
        sprintf(fullMessage,
                "Failed to load parser within server engine: \n \
                Error Message='%s'\
                Error Code='%d'\n \
                Load lib error='%s' \n",
                message->c_str(), (int) dwError, PLATFORM_LOADLIB_ERROR);
        delete( message);
        throw AxisEngineException(SERVER_ENGINE_LOADING_PARSER_FAILED, fullMessage);
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "loadLib", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int XMLParserFactory::unloadLib()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserFactory", "unloadLib", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

    PLATFORM_UNLOADLIB(m_LibHandler);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserFactory", "unloadLib", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

