#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/**
 * This class 
 *
 * @author Susantha Kumara (skumara@virtusa.com, susantha@opensource.lk)
 * @author damitha kumarage (damitha@hsenid.lk, damitha@opensource.lk)
 *
 */

#include "../../platforms/PlatformAutoSense.hpp"

#include "ChannelFactory.hpp"

#include <stdio.h>

#include "HTTPTransportException.hpp"

#include "../../common/AxisTrace.h"

AXIS_CPP_NAMESPACE_START

#define MAXCHANNELS (int)MaxChannelCount

ChannelLibrary *ChannelFactory::m_ChannelLibrary[MAXCHANNELS] = {NULL,NULL};

ChannelFactory::ChannelFactory()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "ChannelFactory", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	for( int iCount = 0; iCount < (int) MaxChannelCount; iCount++)
	{
		m_pLibName[iCount] = NULL;
		m_LibHandler[iCount] = NULL;
		m_pChannel[iCount] = NULL;
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "ChannelFactory", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

ChannelFactory::~ChannelFactory()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "~ChannelFactory", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	for( int eChannelType = 0; eChannelType < (int) MaxChannelCount; eChannelType++)
	{
		UnLoadChannelLibrary( (g_ChannelType) eChannelType);
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "~ChannelFactory", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

IChannel * ChannelFactory::LoadChannelLibrary( g_ChannelType eChannelType, const char * pcLibraryName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "LoadChannelLibrary", this, 2,
					TRACETYPE_DATA, sizeof(g_ChannelType), ((void*)&eChannelType),
					TRACETYPE_STRING, 0, ((void*)&pcLibraryName));	  /* AUTOINSERTED TRACE */
	#endif

	DLHandler	sLibHandler;
	IChannel *	pChannel = NULL;
	int			iLibCount = (int) eChannelType;
    // Additional code added to block reloading of DLL if name has not changed.
	if( m_pLibName[iLibCount] == NULL ||
		strcmp( pcLibraryName, m_pLibName[iLibCount]) != 0)
	{
		sLibHandler = PLATFORM_LOADLIB( pcLibraryName);
		if( !sLibHandler)
		{
                        long dwError = GETLASTERROR
                        string *    message = PLATFORM_GET_ERROR_MESSAGE( dwError);
                        char        fullMessage[1024];
                        sprintf(fullMessage,
                                "Failed to load transport channel within server engine: \n \
                                 Error Message='%s'\
                                 Error Code='%d'\n \
                                 Load lib error='%s' \n",
                                 message->c_str(), (int) dwError, PLATFORM_LOADLIB_ERROR);
                        delete( message);
			throw HTTPTransportException( SERVER_TRANSPORT_LOADING_CHANNEL_FAILED, fullMessage);
		}
		else
		{
			CREATE_OBJECT3 sCreate = (CREATE_OBJECT3) PLATFORM_GETPROCADDR( sLibHandler, CREATE_FUNCTION3);
			DELETE_OBJECT3 sDelete = (DELETE_OBJECT3) PLATFORM_GETPROCADDR( sLibHandler, DELETE_FUNCTION3);
			if (!sCreate || !sDelete)
			{
				PLATFORM_UNLOADLIB( sLibHandler);
				char * pszErrorInfo = new char[ strlen( pcLibraryName) + 1];
				strcpy( pszErrorInfo, pcLibraryName);
				if( eChannelType == UnsecureChannel)
				{
					throw HTTPTransportException( SERVER_TRANSPORT_LOADING_CHANNEL_FAILED, pszErrorInfo);
				}
				else
				{
					throw HTTPTransportException( SERVER_TRANSPORT_LOADING_SSLCHANNEL_FAILED, pszErrorInfo);
				}
			}
#ifdef ENABLE_AXISTRACE
// Load function to do lib level inits
			void (*initializeLibrary) (AxisTraceEntrypoints*);
			initializeLibrary = (void (*)(AxisTraceEntrypoints*))PLATFORM_GETPROCADDR(sLibHandler, "initializeLibrary");
			if( initializeLibrary)
			{
				(*initializeLibrary) (AxisTrace::getTraceEntrypoints());
			}
#endif
            // Additional code added to that when the user wants to load a different
            // library from that which is already loaded, it will now allow the change.
			UnLoadChannelLibrary(eChannelType);
                        
            if (m_pLibName[iLibCount])
            {
                delete [] m_pLibName[iLibCount];
                m_pLibName[iLibCount] = NULL;
            }
			m_pLibName[iLibCount] = new char[ strlen( pcLibraryName) + 1];
			strcpy( m_pLibName[iLibCount], pcLibraryName);
			m_LibHandler[iLibCount] = sLibHandler;
			if( sCreate)
			{
				sCreate( &pChannel);
				m_pChannel[iLibCount] = pChannel;
			}
		}
	}
	else
	{
		pChannel = m_pChannel[iLibCount];
	}
		{
		#ifdef ENABLE_AXISTRACE
			IChannel* traceRet = (pChannel);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "LoadChannelLibrary", this, 0,
					TRACETYPE_POINTER, sizeof(IChannel), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pChannel;
		#endif
	}

}

bool ChannelFactory::UnLoadChannelLibrary( g_ChannelType eChannelType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "UnLoadChannelLibrary", this, 1,
					TRACETYPE_DATA, sizeof(g_ChannelType), ((void*)&eChannelType));	  /* AUTOINSERTED TRACE */
	#endif

	bool	bSuccess = false;
	int		iLibIndex = (int) eChannelType;
	if( m_pChannel[iLibIndex] != NULL)
	{
		if( m_LibHandler[iLibIndex] != NULL)
		{
// If there is a channel library object handle then find the entry point in
// the library to call the channel library destructor (DestroyInstance) which
// in turn calls the channel destructor.  On return from this call, the
// m_pChannel[iLibIndex] object no longer exists.
			DELETE_OBJECT3 sDelete = (DELETE_OBJECT3) PLATFORM_GETPROCADDR( m_LibHandler[iLibIndex], DELETE_FUNCTION3);
			sDelete( m_pChannel[iLibIndex]);
		}
                else
                {
                    // Samisa: somehow, the above block does not seem to deallocate the channel
                    // As a workaround, I delete the channel here and seems this logic is not having any side effects
                    // More importantly I could get rid of the memory leak
                    delete m_pChannel[iLibIndex];
                }
		m_pChannel[iLibIndex] = NULL;
	}
// Delete library name from the library name list.
	if( m_pLibName[iLibIndex] != NULL)
	{
		delete [] m_pLibName[iLibIndex];
		m_pLibName[iLibIndex] = NULL;
	}
// If a channel library object handle is valid, then unload the library and
// reset the handle.
	if( m_LibHandler[iLibIndex] != NULL)
	{
	    PLATFORM_UNLOADLIB( m_LibHandler[iLibIndex]);
		m_LibHandler[iLibIndex] = 0;
		bSuccess = true;
	}
		{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (bSuccess);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "UnLoadChannelLibrary", this, 0,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return bSuccess;
		#endif
	}

}

void ChannelFactory::preloadChannels(char *unsecChannel, char *secChannel)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "preloadChannels", NULL, 2,
					TRACETYPE_STRING, 0, ((void*)&unsecChannel),
					TRACETYPE_STRING, 0, ((void*)&secChannel));	  /* AUTOINSERTED TRACE */
	#endif

	if (unsecChannel && strcmp(unsecChannel,"Unknown")) preloadChannel(UnsecureChannel, unsecChannel);
	if (secChannel && strcmp(secChannel,"Unknown")) preloadChannel(SecureChannel, secChannel);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "preloadChannels", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void ChannelFactory::preloadChannel(g_ChannelType type, const char *pcLibraryName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "preloadChannel", NULL, 2,
					TRACETYPE_DATA, sizeof(g_ChannelType), ((void*)&type),
					TRACETYPE_STRING, 0, ((void*)&pcLibraryName));	  /* AUTOINSERTED TRACE */
	#endif

	int iLibCount = (int)type;
	ChannelLibrary *pCh = new ChannelLibrary();
	pCh->m_Library = PLATFORM_LOADLIB( pcLibraryName);
	if( !pCh->m_Library)
	{
		delete pCh;
           
                long dwError = GETLASTERROR
                string *    message = PLATFORM_GET_ERROR_MESSAGE( dwError);
                char        fullMessage[1024];
                sprintf(fullMessage,
                        "Failed to load transport channel within server engine: \n \
                         Error Message='%s'\
                         Error Code='%d'\n \
                         Load lib error='%s' \n",
                         message->c_str(), (int) dwError, PLATFORM_LOADLIB_ERROR);
                delete( message);
		throw HTTPTransportException( SERVER_TRANSPORT_LOADING_CHANNEL_FAILED, fullMessage);
	}
	pCh->m_Create = (CREATE_OBJECT3) PLATFORM_GETPROCADDR( pCh->m_Library, CREATE_FUNCTION3);
	pCh->m_Delete = (DELETE_OBJECT3) PLATFORM_GETPROCADDR( pCh->m_Library, DELETE_FUNCTION3);
	if (!pCh->m_Create || !pCh->m_Delete)
	{
		PLATFORM_UNLOADLIB( pCh->m_Library);
		delete pCh;
		char * pszErrorInfo = new char[ strlen( pcLibraryName) + 1];
		strcpy( pszErrorInfo, pcLibraryName);
		if( type == UnsecureChannel)
		{
			throw HTTPTransportException( SERVER_TRANSPORT_LOADING_CHANNEL_FAILED, pszErrorInfo);
		}
		else
		{
			throw HTTPTransportException( SERVER_TRANSPORT_LOADING_SSLCHANNEL_FAILED, pszErrorInfo);
		}
	}
#ifdef ENABLE_AXISTRACE
	// Load function to do lib level inits
	void (*initializeLibrary) (AxisTraceEntrypoints*);
	initializeLibrary = (void (*)(AxisTraceEntrypoints*))PLATFORM_GETPROCADDR(pCh->m_Library, "initializeLibrary");
	if( initializeLibrary)
	{
		(*initializeLibrary) (AxisTrace::getTraceEntrypoints());
	}
#endif
	m_ChannelLibrary[iLibCount] = pCh;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "preloadChannel", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

IChannel *ChannelFactory::createChannel(g_ChannelType type) 
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "createChannel", this, 1,
					TRACETYPE_DATA, sizeof(g_ChannelType), ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

	int iLibCount = (int)type;
	IChannel *pChannel = NULL;
	if (m_ChannelLibrary[iLibCount])
	{
		m_ChannelLibrary[iLibCount]->m_Create(&pChannel);
		m_pChannel[iLibCount] = pChannel;
	}
		{
		#ifdef ENABLE_AXISTRACE
			IChannel* traceRet = (pChannel);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "createChannel", this, 0,
					TRACETYPE_POINTER, sizeof(IChannel), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pChannel;
		#endif
	}

}

void ChannelFactory::unloadChannels()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("ChannelFactory", "unloadChannels", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

	for (int i=0; i<(int)MaxChannelCount; i++) 
	{
		if (m_ChannelLibrary[i])
		{
			PLATFORM_UNLOADLIB(m_ChannelLibrary[i]->m_Library);
			delete m_ChannelLibrary[i];
			m_ChannelLibrary[i] = NULL;
		}
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("ChannelFactory", "unloadChannels", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_CPP_NAMESPACE_END

