#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/*
 * @author Susantha Kumara (susantha@opensource.lk)
 */

#ifdef WIN32

#pragma warning (disable : 4786)

#pragma warning (disable : 4101)

#endif

#include "XMLParserAxis.hpp"

#include "AxisInputStream.hpp"
 
XMLParserAxis::XMLParserAxis()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "XMLParserAxis", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	m_bEndElementFollows = false;	
	m_pParser = 0;
    m_pInputStream = 0;
    m_Element.m_pchNamespace = 0;
    m_Element.m_pchNameOrValue = 0;
    m_Element.m_pchAttributes[0] = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "XMLParserAxis", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XMLParserAxis::~XMLParserAxis()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "~XMLParserAxis", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pParser)
        delete m_pParser;
	if (m_pInputStream)
    	delete m_pInputStream;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "~XMLParserAxis", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int XMLParserAxis::setInputStream(AxisIOStream* pInputStream)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "setInputStream", this, 1,
					TRACETYPE_POINTER, sizeof(AxisIOStream), ((void*)&pInputStream));	  /* AUTOINSERTED TRACE */
	#endif

    try 
    {
	    m_bEndElementFollows = false;
        if (m_pInputStream)
            delete m_pInputStream;
        m_pInputStream = new AxisInputStream(pInputStream);
	    //TODO : Reusing the same parser object should be possible. Improve the
	    // parser and then remove the following.
	    if (m_pParser)
		    delete m_pParser;
	    m_pParser = new XmlPullParser(m_pInputStream);
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "setInputStream", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

    }
    catch(XmlPullParserException* pException)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("XMLParserAxis", "setInputStream", this, 0,
					TRACETYPE_POINTER, sizeof(XmlPullParserException), ((void*)&pException));	  /* AUTOINSERTED TRACE */
	#endif

        delete pException;
        throw new AxisParseException();
    }
}

const XML_Ch* XMLParserAxis::getNS4Prefix(const XML_Ch* prefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "getNS4Prefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&prefix));	  /* AUTOINSERTED TRACE */
	#endif

    try
    {
	    	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = (m_pParser->getNamespace4Prefix(prefix));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "getNS4Prefix", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pParser->getNamespace4Prefix(prefix);
		#endif
	}

    }
    catch(XmlPullParserException* pException)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("XMLParserAxis", "getNS4Prefix", this, 0,
					TRACETYPE_POINTER, sizeof(XmlPullParserException), ((void*)&pException));	  /* AUTOINSERTED TRACE */
	#endif

        delete pException;
        throw new AxisParseException();
    }
}

int XMLParserAxis::getStatus()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "getStatus", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	//TODO : implement this correctly
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "getStatus", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

void XMLParserAxis::freeElement()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "freeElement", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (m_bEndElementFollows) // free only attributes list if available.
	{
		m_Element.m_type = END_ELEMENT;
	}
	else // free all inner strings 
	{
		if (m_Element.m_pchNameOrValue)
		{
			free (const_cast<void*>((const void*)m_Element.m_pchNameOrValue));
			m_Element.m_pchNameOrValue = 0;
		}
		if (m_Element.m_pchNamespace)
		{
			free (const_cast<void*>((const void*)m_Element.m_pchNamespace));
			m_Element.m_pchNamespace = 0;
		}
	}
	freeAttributes();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "freeElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XMLParserAxis::freeAttributes()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "freeAttributes", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	for (int ix=0; m_Element.m_pchAttributes[ix]; ix+=3)
	{
		if (m_Element.m_pchAttributes[ix])
		{
			free (const_cast<void*>((const void*)m_Element.m_pchAttributes[ix]));
			if (m_Element.m_pchAttributes[ix+1])
				free (const_cast<void*>((const void*)m_Element.m_pchAttributes[ix+1]));
			if (m_Element.m_pchAttributes[ix+2])
				free (const_cast<void*>((const void*)m_Element.m_pchAttributes[ix+2]));
		}
	}
	m_Element.m_pchAttributes[0] = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "freeAttributes", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XMLParserAxis::setAttributes()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "setAttributes", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	int iAttribIx=0;
	int iAttrib = m_pParser->getAttributeCount();
	m_Element.m_pchAttributes[iAttrib*3]=0;
	for (int ix = 0; ix < iAttrib; ix++) 
	{
		iAttribIx=ix*3;
		m_Element.m_pchAttributes[iAttribIx] = m_pParser->getAttributeName(ix);
		m_Element.m_pchAttributes[iAttribIx+1] =
m_pParser->getAttributeNamespaceUri(ix);
		m_Element.m_pchAttributes[iAttribIx+2] = m_pParser->getAttributeValue(ix);
	}				
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "setAttributes", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const AnyElement* XMLParserAxis::next(bool isCharData)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "next", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&isCharData));	  /* AUTOINSERTED TRACE */
	#endif

	int nType;
    try 
    {
	    freeElement();
	    if (m_bEndElementFollows)
	    {
		    m_bEndElementFollows = false;
		    	{
		#ifdef ENABLE_AXISTRACE
			const AnyElement* traceRet = (&m_Element);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "next", this, 1,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return &m_Element;
		#endif
	}

	    }
	    while (true)
	    {
		    if ((nType= m_pParser->next()) != -1) 
		    {
			    if (!isCharData && (XmlPullParser::Content == nType))
			    { // ignorable white space
				    continue;
			    }			
			    switch (nType)
			    {
			    case XmlPullParser::STag:
				    m_Element.m_type = START_ELEMENT;
				    m_Element.m_pchNameOrValue = m_pParser->getName();
				    m_Element.m_pchNamespace = m_pParser->getNamespaceUri();
				    setAttributes();
				    break;
			    case XmlPullParser::EmptyElemTag:
				    m_Element.m_type = START_ELEMENT;
				    m_Element.m_pchNameOrValue = m_pParser->getName();
				    m_Element.m_pchNamespace = m_pParser->getNamespaceUri();
				    setAttributes();
				    m_bEndElementFollows = true;
				    break;
			    case XmlPullParser::ETag:
				    m_Element.m_type = END_ELEMENT;
				    m_Element.m_pchNameOrValue = m_pParser->getName();
				    m_Element.m_pchNamespace = m_pParser->getNamespaceUri();
				    break;
			    case XmlPullParser::Content:
				    m_Element.m_type = CHARACTER_ELEMENT;
				    m_Element.m_pchNameOrValue = m_pParser->getCharData();
				    break;
			    default:;
			    }
			    	{
		#ifdef ENABLE_AXISTRACE
			const AnyElement* traceRet = (&m_Element);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "next", this, 2,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return &m_Element;
		#endif
	}

		    }
		    else 
		    {
			    	{
		#ifdef ENABLE_AXISTRACE
			const AnyElement* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "next", this, 3,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

		    }
	    }
    }
    catch(XmlPullParserException* pException)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("XMLParserAxis", "next", this, 0,
					TRACETYPE_POINTER, sizeof(XmlPullParserException), ((void*)&pException));	  /* AUTOINSERTED TRACE */
	#endif

        delete pException;
        throw new AxisParseException();
    }
}

const AnyElement* XMLParserAxis::anyNext()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "anyNext", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	//TODO : not implemented. I dont think that this ever need to be implemented
	//because we are not using this any more in Axis.
		{
		#ifdef ENABLE_AXISTRACE
			const AnyElement* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "anyNext", this, 0,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const XML_Ch* XMLParserAxis::getPrefix4NS(const XML_Ch* pcNS)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserAxis", "getPrefix4NS", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcNS));	  /* AUTOINSERTED TRACE */
	#endif

    try
    {
        	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = (m_pParser->getPrefix4Namespace(pcNS));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserAxis", "getPrefix4NS", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pParser->getPrefix4Namespace(pcNS);
		#endif
	}

    }
    catch(XmlPullParserException* pException)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("XMLParserAxis", "getPrefix4NS", this, 0,
					TRACETYPE_POINTER, sizeof(XmlPullParserException), ((void*)&pException));	  /* AUTOINSERTED TRACE */
	#endif

        delete pException;
        throw new AxisParseException();
    }
}

