#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/*
 * @author sanjaya singharage (sanjayas@opensource.lk)
 * @author Susantha Kumara (susantha@opensource.lk, skumara@virtusa.com)
 */

#ifdef WIN32

#pragma warning (disable : 4786)

#endif

#include "../../platforms/PlatformAutoSense.hpp"

#include "XercesHandler.h"

#include <xercesc/sax2/Attributes.hpp>

#include <stdio.h>

XercesHandler::XercesHandler()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "XercesHandler", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_nStatus = AXIS_SUCCESS;
	m_pCurrElement = 0;
    m_pNextElement = new AnyElement;
	m_pPrefixMappingElement = new AnyElement;
    m_bEndElementFollows = false;
    m_pNextElement->m_pchNameOrValue = 0;
    m_pNextElement->m_pchNamespace = 0;
	m_pNextElement->m_pchAttributes[0] = 0;
    m_pPrefixMappingElement->m_pchNameOrValue = 0;
    m_pPrefixMappingElement->m_pchNamespace = 0;
	m_pPrefixMappingElement->m_pchAttributes[0] = 0;
	m_bGetPrefixMappings = false;
	m_bStartElementWaiting = false;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "XercesHandler", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XercesHandler::~XercesHandler()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "~XercesHandler", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	delete m_pNextElement;
	delete m_pPrefixMappingElement;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "~XercesHandler", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::startElement(const XMLCh *const uri,const XMLCh *const 
                                 localname,const XMLCh *const qname,
                                 const Attributes &attrs)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "startElement", this, 4,
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&uri),
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&localname),
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&qname),
					TRACETYPE_DATA, sizeof(Attributes&), ((void*)&attrs));	  /* AUTOINSERTED TRACE */
	#endif

    m_pCurrElement = m_pNextElement;
    m_pNextElement->m_type = START_ELEMENT;
    m_pNextElement->m_type2 = END_ELEMENT;
    m_pNextElement->m_pchNameOrValue = XMLString::transcode(localname);
    m_pNextElement->m_pchNamespace = XMLString::transcode(uri);
    
    unsigned int len = attrs.getLength();
    unsigned int index;
    unsigned int i = 0;
    for (index = 0; index < len*3; index+=3)
    {    
        m_pNextElement->m_pchAttributes[index] = 
            XMLString::transcode(attrs.getLocalName(i));
        m_pNextElement->m_pchAttributes[index+1] = 
            XMLString::transcode(attrs.getURI(i));
        m_pNextElement->m_pchAttributes[index+2] = 
            XMLString::transcode(attrs.getValue(i));
        i++;
    }
    m_pNextElement->m_pchAttributes[len*3]=NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "startElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const XML_Ch* XercesHandler::ns4Prefix(const XML_Ch* prefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "ns4Prefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&prefix));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_NsStack.find(prefix) != m_NsStack.end())
    {
        	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = (m_NsStack[prefix].c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "ns4Prefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_NsStack[prefix].c_str();
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "ns4Prefix", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const XML_Ch* XercesHandler::prefix4NS(const XML_Ch* pcNS)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "prefix4NS", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcNS));	  /* AUTOINSERTED TRACE */
	#endif

    for (map<AxisXMLString, AxisXMLString>::iterator it=m_NsStack.begin();
         it!=m_NsStack.end(); it++)
    {
        if ((*it).second == pcNS)
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = ((*it).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "prefix4NS", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*it).first.c_str();
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "prefix4NS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}
    
void XercesHandler::characters(const XMLCh* const chars, 
                               const unsigned int length)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "characters", this, 2,
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&chars),
					TRACETYPE_UINT, 0, ((void*)&length));	  /* AUTOINSERTED TRACE */
	#endif

    const char *	cp_PreviousNameOrValue = NULL;
    if( m_pCurrElement && m_pCurrElement->m_pchNameOrValue)
    {
        if (m_pCurrElement->m_type == CHARACTER_ELEMENT)
		{
            cp_PreviousNameOrValue = m_pCurrElement->m_pchNameOrValue;
		}
    }
    m_pCurrElement = m_pNextElement;
    m_pNextElement->m_type = CHARACTER_ELEMENT;
// The following code is necessary as it is important to get the correct heap
// (i.e the one that XMLString is using!) when creating a memory object to put
// back into 'm_pNextElement->m_pchNameOrValue'.  By using the XMLString
// function, we can ensure that only the memory belonging to (and thus able to
// destroy) the same segment as XMLString is used.  If you don't understand
// what is going on, don't change it!
    if (cp_PreviousNameOrValue)
    {
		// Get a pointer to the transcoded character.
		char *	pTransChar = XMLString::transcode( chars);
		// Create a dummy string and populate.
		char *	psDummy = new char[ strlen( m_pNextElement->m_pchNameOrValue) +
									strlen( pTransChar) + 1];
		strcpy( psDummy, m_pNextElement->m_pchNameOrValue);
		strcat( psDummy, pTransChar);
		// Create pointer to new Name or Value string.
		char *	pNewNameOrValue = XMLString::replicate( psDummy);
		// Delete the old Name and Value string.
		XMLString::release( const_cast<char**> (&(m_pCurrElement->m_pchNameOrValue)));
		// Assign the new value of Name or Value string.
		m_pNextElement->m_pchNameOrValue = pNewNameOrValue;
		// Clean up.
		delete [] psDummy;
		XMLString::release( &pTransChar);
    }
    else
	{
        m_pNextElement->m_pchNameOrValue = XMLString::transcode( chars);
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "characters", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::ignorableWhitespace(const XMLCh* const chars, 
                                        const unsigned int length)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "ignorableWhitespace", this, 2,
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&chars),
					TRACETYPE_UINT, 0, ((void*)&length));	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "ignorableWhitespace", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::resetDocument()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "resetDocument", this, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "resetDocument", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::warning(const SAXParseException& exception)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "warning", this, 1,
					TRACETYPE_DATA, sizeof(SAXParseException&), ((void*)&exception));	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "warning", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::error(const SAXParseException& exception)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "error", this, 1,
					TRACETYPE_DATA, sizeof(SAXParseException&), ((void*)&exception));	  /* AUTOINSERTED TRACE */
	#endif

    m_nStatus = AXIS_FAIL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "error", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::fatalError(const SAXParseException& exception)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "fatalError", this, 1,
					TRACETYPE_DATA, sizeof(SAXParseException&), ((void*)&exception));	  /* AUTOINSERTED TRACE */
	#endif

    m_nStatus = AXIS_FAIL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "fatalError", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::endElement (const XMLCh *const uri,
                                const XMLCh *const localname,
                                const XMLCh *const qname)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "endElement", this, 3,
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&uri),
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&localname),
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&qname));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pCurrElement && (START_ELEMENT == m_pCurrElement->m_type))
    /* it seems that both startElement and endElement events fired within a
    single parseNext call */
    {
        m_pCurrElement->m_type2 = START_END_ELEMENT;
        m_bEndElementFollows = true;
        	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "endElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    }
    m_pCurrElement = m_pNextElement;
    m_pNextElement->m_type = END_ELEMENT;
	if( m_pNextElement->m_type2 == START_END_ELEMENT)
	{
	    m_pNextElement->m_type2 = END_ELEMENT;
	}
    m_pNextElement->m_pchNameOrValue = XMLString::transcode(localname);
    m_pNextElement->m_pchNamespace = XMLString::transcode(uri);
    m_pNextElement->m_pchAttributes[0] = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "endElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::startPrefixMapping(const XMLCh* const prefix,
                                       const XMLCh* const uri)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "startPrefixMapping", this, 2,
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&prefix),
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&uri));	  /* AUTOINSERTED TRACE */
	#endif

	char* pchPrefix = XMLString::transcode(prefix);
	char* pchUri = XMLString::transcode(uri);
    m_NsStack[pchPrefix] = pchUri;
	/* Store only if interested in prefix mappings */
	if (m_bGetPrefixMappings)
	{
		m_CurrPrefixMappings[pchPrefix] = pchUri;
	}
	else
	{
		XMLString::release(&pchPrefix);
		XMLString::release(&pchUri);
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "startPrefixMapping", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::endPrefixMapping(const XMLCh* const prefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "endPrefixMapping", this, 1,
					TRACETYPE_POINTER, sizeof(XMLCh), ((void*)&prefix));	  /* AUTOINSERTED TRACE */
	#endif

	/* we are not interested in end prefix mapping events in any case */
    char* pchPrefix = XMLString::transcode(prefix);
    m_NsStack.erase(pchPrefix);
	XMLString::release(&pchPrefix);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "endPrefixMapping", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::freeElement()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "freeElement", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pCurrElement)
    {
        if (m_bEndElementFollows)
        /* free only attributes list if available. Next time 
		 * the same m_pNextElement is freed.
		 */
        {
            m_bEndElementFollows = false;
            m_pCurrElement->m_type = END_ELEMENT;
			freeAttributes();
            m_pCurrElement->m_pchAttributes[0] = NULL;
        }
        else
        /* free all inner strings */
        {
			if (m_pCurrElement->m_pchNameOrValue)
			{
				XMLString::release(const_cast<char**>(&(m_pCurrElement->
					m_pchNameOrValue)));
				m_pCurrElement->m_pchNameOrValue = 0;
			}
			if (m_pCurrElement->m_pchNamespace)
			{
				XMLString::release(const_cast<char**>(&(m_pCurrElement->
					m_pchNamespace)));
				m_pCurrElement->m_pchNamespace = 0;
			}
			freeAttributes();
            m_pCurrElement = 0;
        }
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "freeElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::freeAttributes()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "freeAttributes", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	for (int x=0; m_pCurrElement->m_pchAttributes[x]; x+=3)
	{
		if (m_pCurrElement->m_pchAttributes[x])
		{
			XMLString::release(const_cast<char**>(&(m_pCurrElement->
				m_pchAttributes[x])));
			m_pCurrElement->m_pchAttributes[x] = 0;
			if (m_pCurrElement->m_pchAttributes[x+1])
				XMLString::release(const_cast<char**>(&(m_pCurrElement->
				m_pchAttributes[x+1])));
			if (m_pCurrElement->m_pchAttributes[x+2])
				XMLString::release(const_cast<char**>(&(m_pCurrElement->
				m_pchAttributes[x+2])));
		}
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "freeAttributes", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AnyElement* XercesHandler::getAnyElement()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "getAnyElement", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (m_bGetPrefixMappings)
	{
		if (m_CurrPrefixMappings.size() > 0)
		{
			map<const AxisChar*, const AxisChar*>::iterator it = 
				m_CurrPrefixMappings.begin();
			m_pPrefixMappingElement->m_pchNameOrValue = (*it).first;
			m_pPrefixMappingElement->m_pchNamespace = (*it).second;
			m_pPrefixMappingElement->m_type = START_PREFIX;
			m_CurrPrefixMappings.erase(it);
			m_pCurrElement = m_pPrefixMappingElement;
			m_bStartElementWaiting = true;
		}
		else if (m_bStartElementWaiting)
		{
			m_pCurrElement = m_pNextElement;
			m_bStartElementWaiting = false;
		}
	}
		{
		#ifdef ENABLE_AXISTRACE
			AnyElement* traceRet = (m_pCurrElement);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "getAnyElement", this, 0,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pCurrElement;
		#endif
	}

}

//Chinthana:This is the method will use by the XercesPaser to peek head the next element 

const char* XercesHandler::peekNextElementName()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "peekNextElementName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif
		
    if (m_pCurrElement)
			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pCurrElement->m_pchNameOrValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "peekNextElementName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pCurrElement->m_pchNameOrValue;
		#endif
	}

	else 
			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "peekNextElementName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const XML_NODE_TYPE XercesHandler::peekNextElementType()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "peekNextElementType", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (m_pCurrElement)
	{
			{
		#ifdef ENABLE_AXISTRACE
			const XML_NODE_TYPE traceRet = ((const XML_NODE_TYPE) m_pCurrElement->m_type);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "peekNextElementType", this, 1,
					TRACETYPE_DATA, sizeof(XML_NODE_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (const XML_NODE_TYPE) m_pCurrElement->m_type;
		#endif
	}

	}
	else
	{
			{
		#ifdef ENABLE_AXISTRACE
			const XML_NODE_TYPE traceRet = (UNKNOWN);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "peekNextElementType", this, 2,
					TRACETYPE_DATA, sizeof(XML_NODE_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return UNKNOWN;
		#endif
	}

	}
}

//28/04/2005

void XercesHandler::setGetPrefixMappings(bool bValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "setGetPrefixMappings", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&bValue));	  /* AUTOINSERTED TRACE */
	#endif

	m_bGetPrefixMappings = bValue;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "setGetPrefixMappings", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XercesHandler::reset()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XercesHandler", "reset", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_nStatus = AXIS_SUCCESS;
    m_bEndElementFollows = false;
    m_pCurrElement = m_pNextElement;
	freeElement();
	m_pCurrElement = m_pPrefixMappingElement;
	freeElement();
	m_bGetPrefixMappings = false;
	m_bStartElementWaiting = false;	
	m_pCurrElement = 0;
	if (m_CurrPrefixMappings.size() > 0)
	{
		map<const AxisChar*, const AxisChar*>::iterator it = 
			m_CurrPrefixMappings.begin();
		XMLString::release(const_cast<char**>(&((*it).first)));
		XMLString::release(const_cast<char**>(&((*it).second)));
		m_CurrPrefixMappings.erase(it);
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XercesHandler", "reset", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

